@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/ed.decls.h,v 3.11 1991/11/26 04:28:26 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.05/RCS/ed.decls.h,v 3.20 1994/06/19 20:46:44 christos Exp $ */
 /*
  * ed.decls.h: Editor external definitions
  */
@@ -46,10 +46,10 @@ extern	void	DeleteBack		__P((int));
 /*
  * ed.init.c
  */
-extern	void	check_window_size	__P((int));
 #ifdef SIG_WINDOW
+extern	void	check_window_size	__P((int));
 extern	sigret_t window_change		__P((int));
-#endif
+#endif /* SIG_WINDOW */
 extern	int	ed_Setup		__P((int));
 extern	void	ed_Init			__P((void));
 extern	int	Cookedmode		__P((void));
@@ -58,7 +58,7 @@ extern	void	ed_set_tty_eight_bit	__P((void));
 
 extern	void	QuoteModeOn		__P((void));
 extern	void	QuoteModeOff		__P((void));
-extern	void	ResetInLine		__P((void));
+extern	void	ResetInLine		__P((int));
 extern	int	Load_input_line		__P((void));
 
 /*
@@ -91,11 +91,19 @@ extern	void	DeleteChars		__P((int));
 extern	void	TellTC			__P((char *));
 extern	void	SetTC			__P((char *, char *));
 extern	void	EchoTC			__P((Char **));
+extern	int 	SetArrowKeys		__P((Char *, XmapVal *, int));
+extern	int 	IsArrowKey		__P((Char *));
+extern	void	ResetArrowKeys		__P((void));
+extern	void	DefaultArrowKeys	__P((void));
+extern	int 	ClearArrowKeys		__P((Char *));
+extern	void 	PrintArrowKeys		__P((Char *));
 extern	void	BindArrowKeys		__P((void));
 extern	void	Beep			__P((void));
 extern	int	CanWeTab		__P((void));
 extern	void	ChangeSize		__P((int, int));
+#ifdef SIG_WINDOW
 extern	int	GetSize			__P((int *, int *));
+#endif /* SIG_WINDOW */
 extern	void	ClearToBottom		__P((void));
 extern	void	GetTermCaps		__P((void));
 
@@ -115,7 +123,10 @@ extern	CCRETVAL	e_insert		__P((int));
 extern	CCRETVAL	e_newline		__P((int));
 extern	CCRETVAL	e_delprev		__P((int));
 extern	CCRETVAL	e_delnext		__P((int));
-extern	CCRETVAL	e_list_delnext		__P((int));	/* for ^D */
+/* added by mtk@ari.ncl.omron.co.jp (920818) */
+extern	CCRETVAL	e_delnext_eof		__P((int));	
+extern	CCRETVAL	e_delnext_list		__P((int));
+extern	CCRETVAL	e_delnext_list_eof	__P((int));	/* for ^D */
 extern	CCRETVAL	e_toend			__P((int));
 extern	CCRETVAL	e_tobeg			__P((int));
 extern	CCRETVAL	e_charback		__P((int));
@@ -214,6 +225,12 @@ extern	CCRETVAL	v_rchar_back		__P((int));
 extern  CCRETVAL        v_charto_fwd		__P((int));
 extern  CCRETVAL        v_charto_back		__P((int));
 extern  CCRETVAL        e_normalize_path	__P((int));
+extern  CCRETVAL        e_normalize_command	__P((int));
+extern  CCRETVAL        e_stuff_char		__P((int));
+extern  CCRETVAL        e_list_all		__P((int));
+extern  CCRETVAL        e_complete_all		__P((int));
+extern  CCRETVAL        e_complete_fwd		__P((int));
+extern  CCRETVAL        e_complete_back		__P((int));
 
 /*
  * ed.inputl.c
@@ -240,8 +257,9 @@ extern  XmapVal *XmapCmd		__P((int));
 extern	void	 AddXkey		__P((Char *, XmapVal *, int));
 extern	void	 ClearXkey		__P((KEYCMD *, Char *));
 extern	int	 GetXkey		__P((Char *, XmapVal *));
-extern	void	 ResetXmap		__P((int));
+extern	void	 ResetXmap		__P((void));
 extern	int	 DeleteXkey		__P((Char *));
 extern	void	 PrintXkey		__P((Char *));
+extern	int	 printOne		__P((Char *, XmapVal *, int));
 
 #endif /* _h_ed_decls */
