@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.04/RCS/ed.h,v 3.21 1993/04/26 21:13:10 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.05/RCS/ed.h,v 3.23 1994/05/26 13:11:20 christos Exp $ */
 /*
  * ed.h: Editor declarations and globals
  */
@@ -95,6 +95,7 @@ extern KEYCMD NumFuns;		/* number of KEYCMDs in above table */
 #define CC_COMPLETE_ALL		18
 #define CC_COMPLETE_FWD		19
 #define CC_COMPLETE_BACK	20
+#define CC_NORMALIZE_COMMAND	21
 
 typedef union Xmapval {		/* value passed to the Xkey routines */
     KEYCMD cmd;
@@ -146,7 +147,6 @@ EXTERN Char CurrentHistLit;	/* Literal status of current show history line */
 /*
  * These are truly extern
  */
-extern Char PromptBuf[];
 extern int MacroLvl;
 
 EXTERN Char *KeyMacro[MAXMACROLEVELS];
