@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.04/RCS/ed.inputl.c,v 3.35 1993/06/25 21:17:12 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.05/RCS/ed.inputl.c,v 3.39 1994/05/07 18:51:25 christos Exp $ */
 /*
  * ed.inputl.c: Input line handling.
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.inputl.c,v 3.35 1993/06/25 21:17:12 christos Exp $")
+RCSID("$Id: ed.inputl.c,v 3.39 1994/05/07 18:51:25 christos Exp $")
 
 #include "ed.h"
 #include "ed.defns.h"		/* for the function names */
@@ -51,15 +51,34 @@ extern bool MapsAreInited;
 extern bool Tty_raw_mode;
 
 /* mismatched first character */
-static Char mismatch[] = {'!', '\\', '^', '-', '%', '\0'};
+static Char mismatch[] = 
+    {'!', '^' , '\\', '-', '%', '\0', '"', '\'', '`', '\0' };
 
+static	int	Repair		__P((void));
 static	int	GetNextCommand	__P((KEYCMD *, Char *));
 static	int	SpellLine	__P((int));
+static	int	CompleteLine	__P((void));
 static	void	RunCommand	__P((Char *));
-static void 	doeval1		__P((Char **));
+static  void 	doeval1		__P((Char **));
 
 static bool rotate = 0;
 
+
+static int
+Repair()
+{
+    if (NeedsRedraw) {
+	ClearLines();
+	ClearDisp();
+	NeedsRedraw = 0;
+    }
+    Refresh();
+    Argument = 1;
+    DoingArg = 0;
+    curchoice = -1;
+    return LastChar - InputBuf;
+}
+
 /* CCRETVAL */
 int
 Inputl()
@@ -206,11 +225,14 @@ Inputl()
 	case CC_NEWLINE:	/* normal end of line */
 	    curlen = 0;
 	    curchoice = -1;
+	    matchval = 1;
 	    if (crct && (!Strcmp(*(crct->vec), STRcmd) ||
 			 !Strcmp(*(crct->vec), STRall))) {
+                PastBottom();
 		copyn(Origin, InputBuf, INBUFSIZE);
 		SaveChar = LastChar;
 		if (SpellLine(!Strcmp(*(crct->vec), STRcmd)) == 1) {
+                    PastBottom();
 		    copyn(Change, InputBuf, INBUFSIZE);
 		    *Strchr(Change, '\n') = '\0';
 		    CorrChar = LastChar;	/* Save the corrected end */
@@ -239,7 +261,7 @@ Inputl()
 			}
 			else if (ch == 'a') {
 			    xprintf("abort\n");
-			    *LastChar = '\0';
+		            LastChar = InputBuf;   /* Null the current line */
 			    Cursor = LastChar;
 			    printprompt(0, NULL);
 			    Refresh();
@@ -249,44 +271,64 @@ Inputl()
 		    }
 		    flush();
 		}
-	    }			/* end CORRECT code */
-	    tellwhat = 0;	/* just in case */
-	    Hist_num = 0;	/* for the history commands */
-	    num = LastChar - InputBuf;	/* return the number of chars read */
-	    /*
-	     * For continuation lines, we set the prompt to prompt 2
-	     */
-	    printprompt(1, NULL);
+	    } else if (crct && !Strcmp(*(crct->vec), STRcomplete)) {
+                if (LastChar > InputBuf && LastChar[-1] == '\n') {
+                    LastChar[-1] = '\0';
+                    LastChar--;
+                    Cursor = LastChar;
+                }
+                match_unique_match = 1;  /* match unique matches */
+		matchval = CompleteLine();
+                match_unique_match = 0;
+        	curlen = LastChar - InputBuf;
+		if (matchval != 1) {
+                    PastBottom();
+		}
+		if (matchval == 0) {
+		    xprintf("No matching command\n");
+		} else if (matchval == 2) {
+		    xprintf("Ambiguous command\n");
+		}
+	        if (NeedsRedraw) {
+		    ClearLines();
+		    ClearDisp();
+		    NeedsRedraw = 0;
+	        }
+	        Refresh();
+	        Argument = 1;
+	        DoingArg = 0;
+		if (matchval == 1) {
+                    PastBottom();
+                    *LastChar++ = '\n';
+                    *LastChar = '\0';
+		}
+        	curlen = LastChar - InputBuf;
+            }
+	    else
+		PastBottom();
+
+	    if (matchval == 1) {
+	        tellwhat = 0;	/* just in case */
+	        Hist_num = 0;	/* for the history commands */
+		/* return the number of chars read */
+	        num = LastChar - InputBuf;
+	        /*
+	         * For continuation lines, we set the prompt to prompt 2
+	         */
+	        printprompt(1, NULL);
+	    }
 	    break;
 
 	case CC_CORRECT:
 	    if (tenematch(InputBuf, Cursor - InputBuf, SPELL) < 0)
 		Beep();		/* Beep = No match/ambiguous */
-	    if (NeedsRedraw) {
-		ClearLines();
-		ClearDisp();
-		NeedsRedraw = 0;
-	    }
-	    Refresh();
-	    Argument = 1;
-	    DoingArg = 0;
-	    curchoice = -1;
-	    curlen = LastChar - InputBuf;
+	    curlen = Repair();
 	    break;
 
 	case CC_CORRECT_L:
 	    if (SpellLine(FALSE) < 0)
 		Beep();		/* Beep = No match/ambiguous */
-	    if (NeedsRedraw) {
-		ClearLines();
-		ClearDisp();
-		NeedsRedraw = 0;
-	    }
-	    Refresh();
-	    Argument = 1;
-	    DoingArg = 0;
-	    curchoice = -1;
-	    curlen = LastChar - InputBuf;
+	    curlen = Repair();
 	    break;
 
 
@@ -294,7 +336,6 @@ Inputl()
 	case CC_COMPLETE_ALL:
 	case CC_COMPLETE_FWD:
 	case CC_COMPLETE_BACK:
-	    expnum = Cursor - InputBuf;
 	    switch (retval) {
 	    case CC_COMPLETE:
 		fn = RECOGNIZE;
@@ -423,56 +464,31 @@ Inputl()
 	case CC_LIST_GLOB:
 	    if (tenematch(InputBuf, Cursor - InputBuf, GLOB) < 0)
 		Beep();
-	    Refresh();
-	    Argument = 1;
-	    DoingArg = 0;
-	    curchoice = -1;
-	    curlen = LastChar - InputBuf;
+	    curlen = Repair();
 	    break;
 
 	case CC_EXPAND_GLOB:
 	    if (tenematch(InputBuf, Cursor - InputBuf, GLOB_EXPAND) <= 0)
 		Beep();		/* Beep = No match */
-	    if (NeedsRedraw) {
-		ClearLines();
-		ClearDisp();
-		NeedsRedraw = 0;
-	    }
-	    Refresh();
-	    Argument = 1;
-	    DoingArg = 0;
-	    curchoice = -1;
-	    curlen = LastChar - InputBuf;
+	    curlen = Repair();
 	    break;
 
 	case CC_NORMALIZE_PATH:
 	    if (tenematch(InputBuf, Cursor - InputBuf, PATH_NORMALIZE) <= 0)
 		Beep();		/* Beep = No match */
-	    if (NeedsRedraw) {
-		ClearLines();
-		ClearDisp();
-		NeedsRedraw = 0;
-	    }
-	    Refresh();
-	    Argument = 1;
-	    DoingArg = 0;
-	    curchoice = -1;
-	    curlen = LastChar - InputBuf;
+	    curlen = Repair();
 	    break;
 
 	case CC_EXPAND_VARS:
 	    if (tenematch(InputBuf, Cursor - InputBuf, VARS_EXPAND) <= 0)
 		Beep();		/* Beep = No match */
-	    if (NeedsRedraw) {
-		ClearLines();
-		ClearDisp();
-		NeedsRedraw = 0;
-	    }
-	    Refresh();
-	    Argument = 1;
-	    DoingArg = 0;
-	    curchoice = -1;
-	    curlen = LastChar - InputBuf;
+	    curlen = Repair();
+	    break;
+
+	case CC_NORMALIZE_COMMAND:
+	    if (tenematch(InputBuf, Cursor - InputBuf, COMMAND_NORMALIZE) <= 0)
+		Beep();		/* Beep = No match */
+	    curlen = Repair();
 	    break;
 
 	case CC_HELPME:
@@ -744,6 +760,9 @@ SpellLine(cmdonly)
 		Cursor--;
 	    endflag = 0;
 	}
+	/* Obey current history character settings */
+	mismatch[0] = HIST;
+	mismatch[1] = HISTSUB;
 	if (!Strchr(mismatch, *argptr) &&
 	    (!cmdonly || starting_a_command(argptr, InputBuf))) {
 	    switch (tenematch(InputBuf, Cursor - InputBuf, SPELL)) {
@@ -769,3 +788,54 @@ SpellLine(cmdonly)
     return matchval;
 }
 
+/*
+ * CompleteLine - do command completion on the entire command line
+ * (which may have trailing newline).
+ * Return value:
+ *  0: No command matched or failure
+ *  1: One command matched
+ *  2: Several commands matched
+ */
+static int
+CompleteLine()
+{
+    int     endflag, tmatch;
+    Char   *argptr, *OldCursor, *OldLastChar;
+
+    OldLastChar = LastChar;
+    OldCursor = Cursor;
+    argptr = InputBuf;
+    endflag = 1;
+    do {
+	while (ismetahash(*argptr) || iscmdmeta(*argptr))
+	    argptr++;
+	for (Cursor = argptr;
+	     *Cursor != '\0' && ((Cursor != argptr && Cursor[-1] == '\\') ||
+				 (!ismetahash(*Cursor) && !iscmdmeta(*Cursor)));
+	     Cursor++)
+	     continue;
+	if (*Cursor == '\0') {
+	    Cursor = LastChar;
+	    if (LastChar[-1] == '\n')
+		Cursor--;
+	    endflag = 0;
+	}
+	if (!Strchr(mismatch, *argptr) && starting_a_command(argptr, InputBuf)) {
+	    tmatch = tenematch(InputBuf, Cursor - InputBuf, RECOGNIZE);
+	    if (tmatch <= 0) {
+                return 0;
+            } else if (tmatch > 1) {
+                return 2;
+	    }
+	    if (LastChar != OldLastChar) {
+		if (argptr < OldCursor)
+		    OldCursor += (LastChar - OldLastChar);
+		OldLastChar = LastChar;
+	    }
+	}
+	argptr = Cursor;
+    } while (endflag);
+    Cursor = OldCursor;
+    return 1;
+}
+
