@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.04/RCS/sh.set.c,v 3.23 1993/07/03 23:47:53 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.05/RCS/sh.set.c,v 3.25 1994/04/12 15:46:46 christos Exp $ */
 /*
  * sh.set.c: Setting and Clearing of variables
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.set.c,v 3.23 1993/07/03 23:47:53 christos Exp $")
+RCSID("$Id: sh.set.c,v 3.25 1994/04/12 15:46:46 christos Exp $")
 
 #include "ed.h"
 #include "tw.h"
@@ -69,7 +69,7 @@ update_vars(vp)
 	dohash(NULL, NULL);
     }
     else if (eq(vp, STRhistchars)) {
-	register Char *pn = value(vp);
+	register Char *pn = varval(vp);
 
 	HIST = *pn++;
 	HISTSUB = *pn;
@@ -78,14 +78,17 @@ update_vars(vp)
 	HistLit = 1;
     }
     else if (eq(vp, STRuser)) {
-	tsetenv(STRKUSER, value(vp));
-	tsetenv(STRLOGNAME, value(vp));
+	tsetenv(STRKUSER, varval(vp));
+	tsetenv(STRLOGNAME, varval(vp));
     }
     else if (eq(vp, STRwordchars)) {
-	word_chars = value(vp);
+	word_chars = varval(vp);
+    }
+    else if (eq(vp, STRloginsh)) {
+	loginsh = 1;
     }
     else if (eq(vp, STRsymlinks)) {
-	register Char *pn = value(vp);
+	register Char *pn = varval(vp);
 
 	if (eq(pn, STRignore))
 	    symlinks = SYM_IGNORE;
@@ -100,7 +103,7 @@ update_vars(vp)
 #ifdef DOESNT_WORK_RIGHT
 	register Char *cp;
 #endif /* DOESNT_WORK_RIGHT */
-	tsetenv(STRKTERM, value(vp));
+	tsetenv(STRKTERM, varval(vp));
 #ifdef DOESNT_WORK_RIGHT
 	cp = getenv("TERMCAP");
 	if (cp && (*cp != '/'))	/* if TERMCAP and not a path */
@@ -112,7 +115,7 @@ update_vars(vp)
     else if (eq(vp, STRhome)) {
 	register Char *cp;
 
-	cp = Strsave(value(vp));	/* get the old value back */
+	cp = Strsave(varval(vp));	/* get the old value back */
 
 	/*
 	 * convert to cononical pathname (possibly resolving symlinks)
@@ -132,7 +135,7 @@ update_vars(vp)
 	/* PWP: add more stuff in here later */
     }
     else if (eq(vp, STRshlvl)) {
-	tsetenv(STRKSHLVL, value(vp));
+	tsetenv(STRKSHLVL, varval(vp));
     }
     else if (eq(vp, STRbackslash_quote)) {
 	bslash_quote = 1;
@@ -356,7 +359,7 @@ dolet(v, dummy)
 	    asx(vp, subscr, operate(op, gv->vec[subscr - 1], p));
 	}
 	else
-	    set(vp, operate(op, value(vp), p), VAR_READWRITE);
+	    set(vp, operate(op, varval(vp), p), VAR_READWRITE);
 	update_vars(vp);
 	xfree((ptr_t) vp);
 	if (c != '=')
@@ -613,6 +616,8 @@ unset(v, c)
     }
     if (adrof(STRhistlit) == 0)
 	HistLit = 0;
+    if (adrof(STRloginsh) == 0)
+	loginsh = 0;
     if (adrof(STRwordchars) == 0)
 	word_chars = STR_WORD_CHARS;
     if (adrof(STRedit) == 0)
