@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.04/RCS/tc.alloc.c,v 3.23 1993/07/03 23:47:53 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.05/RCS/tc.alloc.c,v 3.24 1994/06/18 19:48:50 christos Exp $ */
 /*
  * tc.alloc.c (Caltech) 2/21/82
  * Chris Kingsley, kingsley@cit-20.
@@ -44,7 +44,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.alloc.c,v 3.23 1993/07/03 23:47:53 christos Exp $")
+RCSID("$Id: tc.alloc.c,v 3.24 1994/06/18 19:48:50 christos Exp $")
 
 static char   *memtop = NULL;		/* PWP: top of current memory */
 static char   *membot = NULL;		/* PWP: bottom of allocatable memory */
@@ -211,8 +211,7 @@ malloc(nbytes)
     /*
      * Record allocated size of block and bound space with magic numbers.
      */
-    if (nbytes <= 0x10000)
-	p->ov_size = nbytes - 1;
+    p->ov_size = (p->ov_index <= 13) ? nbytes - 1 : 0;
     p->ov_rmagic = RMAGIC;
     *((U_int *) (((caddr_t) p) + nbytes - RSLOP)) = RMAGIC;
 #endif
@@ -397,8 +396,16 @@ realloc(cp, nbytes)
 
     /* avoid the copy if same size block */
     if (was_alloced && (onb <= (U_int) (1 << (i + 3))) && 
-	(onb > (U_int) (1 << (i + 2))))
+	(onb > (U_int) (1 << (i + 2)))) {
+#ifdef RCHECK
+	/* JMR: formerly this wasn't updated ! */
+	nbytes = MEMALIGN(MEMALIGN(sizeof(union overhead))+nbytes+RSLOP);
+	*((U_int *) (((caddr_t) op) + nbytes - RSLOP)) = RMAGIC;
+	op->ov_rmagic = RMAGIC;
+	op->ov_size = (op->ov_index <= 13) ? nbytes - 1 : 0;
+#endif
 	return ((memalign_t) cp);
+    }
     if ((res = malloc(nbytes)) == NULL)
 	return ((memalign_t) NULL);
     if (cp != res) {		/* common optimization */
