@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.02/RCS/tc.func.c,v 3.29 1992/04/10 16:38:09 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.05/RCS/tc.func.c,v 3.57 1994/05/26 13:11:20 christos Exp $ */
 /*
  * tc.func.c: New tcsh builtins.
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.func.c,v 3.29 1992/04/10 16:38:09 christos Exp $")
+RCSID("$Id: tc.func.c,v 3.57 1994/05/26 13:11:20 christos Exp $")
 
 #include "ed.h"
 #include "ed.defns.h"		/* for the function names */
@@ -47,8 +47,18 @@ RCSID("$Id: tc.func.c,v 3.29 1992/04/10 16:38:09 christos Exp $")
 # include <hesiod.h>
 #endif /* HESIOD */
 
-extern time_t t_period;
+#ifdef AFS
+#define PASSMAX 16
+#include <afs/kautils.h>
+long ka_UserAuthenticateGeneral();
+#else
+#define PASSMAX 8
+#endif /* AFS */
+
+#ifdef TESLA
 extern int do_logout;
+#endif /* TESLA */
+extern time_t t_period;
 extern int just_signaled;
 static bool precmd_active = 0;
 static bool periodic_active = 0;
@@ -174,9 +184,10 @@ static void
 Reverse(s)
     Char   *s;
 {
-    int     c, i, j;
+    Char   c;
+    int     i, j;
 
-    for (i = 0, j = Strlen(s) - 1; i < j; i++, j--) {
+    for (i = 0, j = (int) Strlen(s) - 1; i < j; i++, j--) {
 	c = s[i];
 	s[i] = s[j];
 	s[j] = c;
@@ -193,6 +204,7 @@ dolist(v, c)
     int     i, k;
     struct stat st;
 
+    USE(c);
     if (*++v == NULL) {
 	(void) t_search(STRNULL, NULL, LIST, 0, TW_ZERO, 0, STRNULL, 0);
 	return;
@@ -214,10 +226,11 @@ dolist(v, c)
 	 * We cannot process a flag therefore we let ls do it right.
 	 */
 	static Char STRls[] = {'l', 's', '\0'};
-	static Char STRmCF[] = {'-', 'C', 'F', '\0'};
+	static Char STRmCF[] = {'-', 'C', 'F', '\0', '\0' };
 	struct command *t;
 	struct wordent cmd, *nextword, *lastword;
 	Char   *cp;
+	struct varent *vp;
 
 #ifdef BSDSIGS
 	sigmask_t omask = 0;
@@ -225,12 +238,24 @@ dolist(v, c)
 	if (setintr)
 	    omask = sigblock(sigmask(SIGINT)) & ~sigmask(SIGINT);
 #else /* !BSDSIGS */
-	sighold(SIGINT);
+	(void) sighold(SIGINT);
 #endif /* BSDSIGS */
 	if (seterr) {
 	    xfree((ptr_t) seterr);
 	    seterr = NULL;
 	}
+
+	/* Look at showdots, to add -A to the flags if necessary */
+	if ((vp = adrof(STRshowdots)) != NULL) {
+	    if (vp->vec[0][0] == '-' && vp->vec[0][1] == 'A' &&
+		vp->vec[0][2] == '\0')
+		STRmCF[3] = 'A';
+	    else
+		STRmCF[3] = '\0';
+	}
+	else
+	    STRmCF[3] = '\0';
+
 	cmd.word = STRNULL;
 	lastword = &cmd;
 	nextword = (struct wordent *) xcalloc(1, sizeof cmd);
@@ -274,8 +299,7 @@ dolist(v, c)
 	Char   *dp, *tmp, buf[MAXPATHLEN];
 
 	for (k = 0, i = 0; v[k] != NULL; k++) {
-	    tmp = dnormalize(v[k], symlinks == SYM_IGNORE || 
-				   symlinks == SYM_EXPAND);
+	    tmp = dnormalize(v[k], symlinks == SYM_IGNORE);
 	    dp = &tmp[Strlen(tmp) - 1];
 	    if (*dp == '/' && dp != tmp)
 #ifdef apollo
@@ -288,7 +312,7 @@ dolist(v, c)
 			xputchar('\n');
 		    print_by_column(STRNULL, &v[i], k - i, FALSE);
 		}
-		xprintf("%s: %s.\n", short2str(tmp), strerror(errno));
+		xprintf("%S: %s.\n", tmp, strerror(errno));
 		i = k + 1;
 	    }
 	    else if (S_ISDIR(st.st_mode)) {
@@ -301,7 +325,7 @@ dolist(v, c)
 		}
 		if (k != 0 && v[1] != NULL)
 		    xputchar('\n');
-		xprintf("%s:\n", short2str(tmp));
+		xprintf("%S:\n", tmp);
 		for (cp = tmp, dp = buf; *cp; *dp++ = (*cp++ | QUOTE))
 		    continue;
 		if (dp[-1] != (Char) ('/' | QUOTE))
@@ -336,7 +360,7 @@ dotelltc(v, c)
     register Char **v;
     struct command *c;
 {
-
+    USE(c);
     if (!GotTermCaps)
 	GetTermCaps();
 
@@ -378,6 +402,42 @@ dosettc(v, c)
  *  West-Germany
  * Thanks!!
  */
+int
+cmd_expand(cmd, str)
+    Char *cmd;
+    Char *str;
+{
+    struct wordent lexp[3];
+    struct varent *vp;
+    int rv = TRUE;
+
+    lexp[0].next = &lexp[1];
+    lexp[1].next = &lexp[2];
+    lexp[2].next = &lexp[0];
+
+    lexp[0].prev = &lexp[2];
+    lexp[1].prev = &lexp[0];
+    lexp[2].prev = &lexp[1];
+
+    lexp[0].word = STRNULL;
+    lexp[2].word = STRret;
+
+    if ((vp = adrof1(cmd, &aliases)) != NULL) {
+	if (str == NULL) {
+	    xprintf("%S: \t aliased to ", cmd);
+	    blkpr(vp->vec);
+	    xputchar('\n');
+	}
+	else 
+	    blkexpand(vp->vec, str);
+    }
+    else {
+	lexp[1].word = cmd;
+	rv = tellmewhat(lexp, str);
+    }
+    return rv;
+}
+
 
 /*ARGSUSED*/
 void
@@ -385,31 +445,34 @@ dowhich(v, c)
     register Char **v;
     struct command *c;
 {
-    struct wordent lex[3];
-    struct varent *vp;
+    USE(c);
 
-    lex[0].next = &lex[1];
-    lex[1].next = &lex[2];
-    lex[2].next = &lex[0];
-
-    lex[0].prev = &lex[2];
-    lex[1].prev = &lex[0];
-    lex[2].prev = &lex[1];
-
-    lex[0].word = STRNULL;
-    lex[2].word = STRret;
-
-    while (*++v) {
-	if ((vp = adrof1(*v, &aliases)) != NULL) {
-	    xprintf("%s: \t aliased to ", short2str(*v));
-	    blkpr(vp->vec);
-	    xputchar('\n');
-	}
-	else {
-	    lex[1].word = *v;
-	    tellmewhat(lex);
-	}
+#ifdef notdef
+    /* 
+     * We don't want to glob dowhich args because we lose quoteing
+     * E.g. which \ls if ls is aliased will not work correctly if
+     * we glob here.
+     */
+    gflag = 0, tglob(v);
+    if (gflag) {
+	v = globall(v);
+	if (v == 0)
+	    stderror(ERR_NAME | ERR_NOMATCH);
     }
+    else {
+	v = gargv = saveblk(v);
+	trim(v);
+    }
+#endif
+
+    while (*++v) 
+	(void) cmd_expand(*v, NULL);
+
+#ifdef notdef
+    /* Again look at the comment above; since we don't glob, we don't free */
+    if (gargv)
+	blkfree(gargv), gargv = 0;
+#endif
 }
 
 /* PWP: a hack to start up your stopped editor on a single keystroke */
@@ -436,8 +499,10 @@ find_stop_ed()
     else 
 	vp = "vi";
 
-    vpl = strlen(vp);
-    epl = strlen(ep);
+    for (vpl = 0; vp[vpl] && !Isspace(vp[vpl]); vpl++)
+	continue;
+    for (epl = 0; ep[epl] && !Isspace(ep[epl]); epl++)
+	continue;
 
     if (pcurrent == NULL)	/* see if we have any jobs */
 	return NULL;		/* nope */
@@ -471,9 +536,9 @@ fg_proc_entry(pp)
 #ifdef BSDSIGS
     sigmask_t omask;
 #endif
-    jmp_buf osetexit;
+    jmp_buf_t osetexit;
     bool    ohaderr;
-    bool    oGettingInput;
+    Char    oGettingInput;
 
     getexit(osetexit);
 
@@ -490,7 +555,7 @@ fg_proc_entry(pp)
     if (setexit() == 0) {	/* come back here after pjwait */
 	pendjob();
 	pstart(pp, 1);		/* found it. */
-	alarm(0);		/* No autologout */
+	(void) alarm(0);	/* No autologout */
 	pjwait(pp);
     }
     setalarm(1);		/* Autologout back on */
@@ -510,7 +575,7 @@ static char *
 xgetpass(prm)
     char *prm;
 {
-    static char pass[9];
+    static char pass[PASSMAX + 1];
     int fd, i;
     sigret_t (*sigint)();
 
@@ -523,7 +588,7 @@ xgetpass(prm)
     for (i = 0;;)  {
 	if (read(fd, &pass[i], 1) < 1 || pass[i] == '\n') 
 	    break;
-	if (i < 8)
+	if (i < PASSMAX)
 	    i++;
     }
 	
@@ -562,8 +627,8 @@ auto_lock()
 
 # define XCRYPT(a, b) crypt16(a, b)
 
-    if ((pw = getpwuid(geteuid())) != NULL &&	/* effective user passwd  */
-        (apw = getauthuid(geteuid())) != NULL) 	/* enhanced ultrix passwd */
+    if ((pw = getpwuid(euid)) != NULL &&	/* effective user passwd  */
+        (apw = getauthuid(euid)) != NULL) 	/* enhanced ultrix passwd */
 	srpp = apw->a_password;
 
 #endif /* PW_AUTH && !XCRYPT */
@@ -575,7 +640,7 @@ auto_lock()
 
 # define XCRYPT(a, b) crypt(a, b)
 
-    if ((pw = getpwuid(geteuid())) != NULL &&	/* effective user passwd  */
+    if ((pw = getpwuid(euid)) != NULL &&	/* effective user passwd  */
 	(spw = getspnam(pw->pw_name)) != NULL)	/* shadowed passwd	  */
 	srpp = spw->sp_pwdp;
 
@@ -586,7 +651,7 @@ auto_lock()
 
 #define XCRYPT(a, b) crypt(a, b)
 
-    if ((pw = getpwuid(geteuid())) != NULL)	/* effective user passwd  */
+    if ((pw = getpwuid(euid)) != NULL)	/* effective user passwd  */
 	srpp = pw->pw_passwd;
 
 #endif /* !XCRYPT */
@@ -606,10 +671,33 @@ auto_lock()
     xputchar('\n'); 
     for (i = 0; i < 5; i++) {
 	char *crpp, *pp;
+#ifdef AFS
+	char *afsname;
+	Char *safs;
+
+	if ((safs = varval(STRafsuser)) != STRNULL)
+	    afsname = short2str(safs);
+	else
+	    if ((afsname = getenv("AFSUSER")) == NULL)
+	        afsname = pw->pw_name;
+#endif
 	pp = xgetpass("Password:"); 
 
 	crpp = XCRYPT(pp, srpp);
-	if (strcmp(crpp, srpp) == 0) {
+	if ((strcmp(crpp, srpp) == 0)
+#ifdef AFS
+	    || (ka_UserAuthenticateGeneral(KA_USERAUTH_VERSION,
+					   afsname,     /* name */
+					   NULL,        /* instance */
+					   NULL,        /* realm */
+					   pp,          /* password */
+					   0,           /* lifetime */
+					   0, 0,         /* spare */
+					   NULL)        /* reason */
+	    == 0)
+#endif /* AFS */
+	    ) {
+	    (void) memset(pp, 0, PASSMAX);
 	    if (GettingInput && !just_signaled) {
 		(void) Rawmode();
 		ClearLines();	
@@ -634,11 +722,12 @@ auto_logout()
     if (editing)
 	(void) Cookedmode();
     (void) close(SHIN);
-    set(STRlogout, Strsave(STRautomatic));
+    set(STRlogout, Strsave(STRautomatic), VAR_READWRITE);
     child = 1;
 #ifdef TESLA
     do_logout = 1;
 #endif /* TESLA */
+    GettingInput = FALSE; /* make flush() work to write hist files. Huber*/
     goodbye(NULL, NULL);
 }
 
@@ -785,8 +874,8 @@ period_cmd()
     }
     periodic_active = 1;
     if (!whyles && adrof1(STRperiodic, &aliases)) {
-	vp = value(STRtperiod);
-	if (vp == NULL)
+	vp = varval(STRtperiod);
+	if (vp == STRNULL)
 	    return;
 	interval = getn(vp);
 	(void) time(&t);
@@ -816,7 +905,8 @@ aliasrun(cnt, s1, s2)
 {
     struct wordent w, *new1, *new2;	/* for holding alias name */
     struct command *t = NULL;
-    jmp_buf osetexit;
+    jmp_buf_t osetexit;
+    int status;
 
     getexit(osetexit);
     if (seterr) {
@@ -840,6 +930,9 @@ aliasrun(cnt, s1, s2)
 	new1->prev = new2->next = &w;
     }
 
+    /* Save the old status */
+    status = getn(varval(STRstatus));
+
     /* expand aliases like process() does. */
     alias(&w);
     /* build a syntax tree for the command. */
@@ -848,6 +941,8 @@ aliasrun(cnt, s1, s2)
 	stderror(ERR_OLD);
 
     psavejob();
+
+
     /* catch any errors here */
     if (setexit() == 0)
 	/* execute the parse tree. */
@@ -887,6 +982,8 @@ aliasrun(cnt, s1, s2)
     resexit(osetexit);
     prestjob();
     pendjob();
+    /* Restore status */
+    set(STRstatus, putn(status), VAR_READWRITE);
 }
 
 void
@@ -900,13 +997,13 @@ setalarm(lck)
 
     if ((vp = adrof(STRautologout)) != NULL) {
 	if ((cp = vp->vec[0]) != 0) {
-	    if ((logout_time = atoi(short2str(cp)) * 60) > 0) {
+	    if ((logout_time = (unsigned) atoi(short2str(cp)) * 60) > 0) {
 		alrm_time = logout_time;
 		alm_fun = auto_logout;
 	    }
 	}
 	if ((cp = vp->vec[1]) != 0) {
-	    if ((lock_time = atoi(short2str(cp)) * 60) > 0) {
+	    if ((lock_time = (unsigned) atoi(short2str(cp)) * 60) > 0) {
 		if (lck) {
 		    if (alrm_time == 0 || lock_time < alrm_time) {
 			alrm_time = lock_time;
@@ -950,7 +1047,7 @@ rmstar(cp)
     if (!adrof(STRrmstar))
 	return;
 #ifdef RMDEBUG
-    tag = value(STRrmdebug);
+    tag = varval(STRrmdebug);
 #endif /* RMDEBUG */
     we = cp->next;
     while (*we->word == ';' && we != cp)
@@ -1019,7 +1116,7 @@ rmstar(cp)
     if (*tag) {
 	xprintf("command line now is:\n");
 	for (we = cp->next; we != cp; we = we->next)
-	    xprintf("%s ", short2str(we->word));
+	    xprintf("%S ", we->word);
     }
 #endif /* RMDEBUG */
     return;
@@ -1047,10 +1144,10 @@ continue_jobs(cp)
 
 
 #ifdef CNDEBUG
-    tag = value(STRcndebug);
+    tag = varval(STRcndebug);
 #endif /* CNDEBUG */
-    continue_list = value(STRcontinue);
-    continue_args_list = value(STRcontinue_args);
+    continue_list = varval(STRcontinue);
+    continue_args_list = varval(STRcontinue_args);
     if (*continue_list == '\0' && *continue_args_list == '\0')
 	return;
 
@@ -1094,8 +1191,7 @@ continue_jobs(cp)
     if (*tag) {
 	xprintf("command line now is:\n");
 	for (we = cp->next; we != cp; we = we->next)
-	    xprintf("%s ",
-		    short2str(we->word));
+	    xprintf("%S ", we->word);
     }
 #endif /* CNDEBUG */
     return;
@@ -1104,23 +1200,23 @@ continue_jobs(cp)
 /* The actual "aliasing" of for backgrounds() is done here
    with the aid of insert_we().   */
 static void
-insert(plist, file_args)
-    struct wordent *plist;
+insert(pl, file_args)
+    struct wordent *pl;
     bool    file_args;
 {
     struct wordent *now, *last;
     Char   *cmd, *bcmd, *cp1, *cp2;
     int     cmd_len;
     Char   *pause = STRunderpause;
-    int     p_len = Strlen(pause);
+    int     p_len = (int) Strlen(pause);
 
-    cmd_len = Strlen(plist->word);
+    cmd_len = (int) Strlen(pl->word);
     cmd = (Char *) xcalloc(1, (size_t) ((cmd_len + 1) * sizeof(Char)));
-    (void) Strcpy(cmd, plist->word);
+    (void) Strcpy(cmd, pl->word);
 /* Do insertions at beginning, first replace command word */
 
     if (file_args) {
-	now = plist;
+	now = pl;
 	xfree((ptr_t) now->word);
 	now->word = (Char *) xcalloc(1, (size_t) (5 * sizeof(Char)));
 	(void) Strcpy(now->word, STRecho);
@@ -1128,7 +1224,7 @@ insert(plist, file_args)
 	now = (struct wordent *) xcalloc(1, (size_t) sizeof(struct wordent));
 	now->word = (Char *) xcalloc(1, (size_t) (6 * sizeof(Char)));
 	(void) Strcpy(now->word, STRbackqpwd);
-	insert_we(now, plist);
+	insert_we(now, pl);
 
 	for (last = now; *last->word != '\n' && *last->word != ';';
 	     last = last->next)
@@ -1176,7 +1272,7 @@ insert(plist, file_args)
     else {
 	struct wordent *del;
 
-	now = plist;
+	now = pl;
 	xfree((ptr_t) now->word);
 	now->word = (Char *) xcalloc(1, 
 				     (size_t) ((cmd_len + 2) * sizeof(Char)));
@@ -1186,7 +1282,7 @@ insert(plist, file_args)
 	while ((*cp1++ = *cp2++) != '\0')
 	    continue;
 	for (now = now->next;
-	     *now->word != '\n' && *now->word != ';' && now != plist;) {
+	     *now->word != '\n' && *now->word != ';' && now != pl;) {
 	    now->prev->next = now->next;
 	    now->next->prev = now->prev;
 	    xfree((ptr_t) now->word);
@@ -1352,7 +1448,7 @@ gettilde(us)
      */
     tcache[tlength].user = Strsave(us);
     tcache[tlength].home = hd;
-    tcache[tlength++].hlen = Strlen(hd);
+    tcache[tlength++].hlen = (int) Strlen(hd);
 
     qsort((ptr_t) tcache, (size_t) tlength, sizeof(struct tildecache),
 	  (int (*) __P((const void *, const void *))) tildecompare);
@@ -1391,21 +1487,22 @@ getusername(hm)
 	tcache = NULL;
 	return NULL;
     }
-    if (((h = value(STRhome)) != NULL) &&
-	(Strncmp(p = *hm, h, j = Strlen(h)) == 0) &&
+    if (((h = varval(STRhome)) != STRNULL) &&
+	(Strncmp(p = *hm, h, (size_t) (j = (int) Strlen(h))) == 0) &&
 	(p[j] == '/' || p[j] == '\0')) {
 	*hm = &p[j];
 	return STRNULL;
     }
     for (i = 0; i < tlength; i++)
-	if ((Strncmp(p = *hm, tcache[i].home, j = tcache[i].hlen) == 0) &&
-	    (p[j] == '/' || p[j] == '\0')) {
+	if ((Strncmp(p = *hm, tcache[i].home, (size_t)
+	    (j = tcache[i].hlen)) == 0) && (p[j] == '/' || p[j] == '\0')) {
 	    *hm = &p[j];
 	    return tcache[i].user;
 	}
     return NULL;
 }
 
+#ifdef OBSOLETE
 /*
  * PWP: read a bunch of aliases out of a file QUICKLY.  The format
  *  is almost the same as the result of saying "alias > FILE", except
@@ -1418,17 +1515,18 @@ doaliases(v, c)
     Char  **v;
     struct command *c;
 {
-    jmp_buf oldexit;
+    jmp_buf_t oldexit;
     Char  **vec, *lp;
     int     fd;
     Char    buf[BUFSIZE], line[BUFSIZE];
     char    tbuf[BUFSIZE + 1], *tmp;
     extern bool output_raw;	/* PWP: in sh.print.c */
 
+    USE(c);
     v++;
     if (*v == 0) {
 	output_raw = 1;
-	plist(&aliases);
+	plist(&aliases, VAR_ALL);
 	output_raw = 0;
 	return;
     }
@@ -1460,7 +1558,7 @@ doaliases(v, c)
 		    if ((n = read(fd, tbuf, BUFSIZE)) <= 0)
 			goto eof;
 		    for (i = 0; i < n; i++)
-			buf[i] = tbuf[i];
+  			buf[i] = (Char) tbuf[i];
 		    p = buf;
 		}
 		n--;
@@ -1478,7 +1576,7 @@ doaliases(v, c)
 					    (2 * sizeof(Char **)));
 		    vec[0] = Strsave(lp);
 		    vec[1] = NULL;
-		    setq(strip(line), vec, &aliases);
+		    setq(strip(line), vec, &aliases, VAR_READWRITE);
 		    break;
 		}
 	    }
@@ -1492,6 +1590,7 @@ eof:
 	blkfree(gargv), gargv = 0;
     resexit(oldexit);
 }
+#endif /* OBSOLETE */
 
 
 /*
@@ -1505,22 +1604,303 @@ shlvl(val)
 
     if ((cp = getenv("SHLVL")) != NULL) {
 
-	val += atoi(cp);
+	if (loginsh)
+	    val = 1;
+	else
+	    val += atoi(cp);
 
 	if (val <= 0) {
-	    unsetv(STRshlvl);
-	    Unsetenv(STRSHLVL);
+	    if (adrof(STRshlvl) != NULL)
+		unsetv(STRshlvl);
+	    Unsetenv(STRKSHLVL);
 	}
 	else {
 	    Char    buff[BUFSIZE];
 
 	    Itoa(val, buff);
-	    set(STRshlvl, Strsave(buff));
-	    Setenv(STRSHLVL, buff);
+	    set(STRshlvl, Strsave(buff), VAR_READWRITE);
+	    tsetenv(STRKSHLVL, buff);
 	}
     }
     else {
-	set(STRshlvl, SAVE("1"));
-	Setenv(STRSHLVL, str2short("1"));
+	set(STRshlvl, SAVE("1"), VAR_READWRITE);
+	tsetenv(STRKSHLVL, str2short("1"));
     }
 }
+
+
+/* fixio():
+ *	Try to recover from a read error
+ */
+int
+fixio(fd, e)
+    int fd, e;
+{
+    switch (e) {
+    case -1:	/* Make sure that the code is reachable */
+
+#ifdef EWOULDBLOCK
+    case EWOULDBLOCK:
+# define FDRETRY
+#endif /* EWOULDBLOCK */
+
+#if defined(POSIX) && defined(EAGAIN)
+# if !defined(EWOULDBLOCK) || EWOULDBLOCK != EAGAIN
+    case EAGAIN:
+#  define FDRETRY
+# endif /* !EWOULDBLOCK || EWOULDBLOCK != EAGAIN */
+#endif /* POSIX && EAGAIN */
+
+	e = 0;
+#ifdef FDRETRY
+# ifdef F_SETFL
+/*
+ * Great! we have on suns 3 flavors and 5 names...
+ * I hope that will cover everything.
+ * I added some more defines... many systems have different defines.
+ * Rather than dealing with getting the right includes, we'll just
+ * cover all the known possibilities here.  -- sterling@netcom.com
+ */
+#  ifndef O_NONBLOCK
+#   define O_NONBLOCK 0
+#  endif /* O_NONBLOCK */
+#  ifndef O_NDELAY
+#   define O_NDELAY 0
+#  endif /* O_NDELAY */
+#  ifndef FNBIO
+#   define FNBIO 0
+#  endif /* FNBIO */
+#  ifndef _FNBIO
+#   define _FNBIO 0
+#  endif /* _FNBIO */
+#  ifndef FNONBIO
+#   define FNONBIO 0
+#  endif /* FNONBIO */
+#  ifndef FNONBLOCK
+#   define FNONBLOCK 0
+#  endif /* FNONBLOCK */
+#  ifndef _FNONBLOCK
+#   define _FNONBLOCK 0
+#  endif /* _FNONBLOCK */
+#  ifndef FNDELAY
+#   define FNDELAY 0
+#  endif /* FNDELAY */
+#  ifndef _FNDELAY
+#   define _FNDELAY 0
+#  endif /* _FNDELAY */
+#  ifndef FNDLEAY	/* Some linux versions have this typo */
+#   define FNDLEAY 0
+#  endif /* FNDLEAY */
+	if ((e = fcntl(fd, F_GETFL, 0)) == -1)
+	    return -1;
+
+	e &= ~(O_NDELAY|O_NONBLOCK|FNBIO|_FNBIO|FNONBIO|FNONBLOCK|_FNONBLOCK|
+	       FNDELAY|_FNDELAY|FNDLEAY);	/* whew! */
+	if (fcntl(fd, F_SETFL, e) == -1)
+	    return -1;
+	else 
+	    e = 1;
+# endif /* F_SETFL */
+
+# ifdef FIONBIO
+	e = 0;
+	if (ioctl(fd, FIONBIO, (ioctl_t) &e) == -1)
+	    return -1;
+	else
+	    e = 1;
+# endif	/* FIONBIO */
+
+#endif /* FDRETRY */
+	return e ? 0 : -1;
+
+    case EINTR:
+	return 0;
+
+    default:
+	return -1;
+    }
+}
+
+/* collate():
+ *	String collation
+ */
+int
+collate(a, b)
+    const Char *a;
+    const Char *b;
+{
+    int rv;
+#ifdef SHORT_STRINGS
+    /* This strips the quote bit as a side effect */
+    char *sa = strsave(short2str(a));
+    char *sb = strsave(short2str(b));
+#else
+    char *sa = strip(strsave(a));
+    char *sb = strip(strsave(b));
+#endif /* SHORT_STRINGS */
+
+#if defined(NLS) && !defined(NOSTRCOLL)
+    errno = 0;	/* strcoll sets errno, another brain-damage */
+
+    rv = strcoll(sa, sb);
+
+    /*
+     * We should be checking for errno != 0, but some systems
+     * forget to reset errno to 0. So we only check for the 
+     * only documented valid errno value for strcoll [EINVAL]
+     */
+    if (errno == EINVAL) {
+	xfree((ptr_t) sa);
+	xfree((ptr_t) sb);
+	stderror(ERR_SYSTEM, "strcoll", strerror(errno));
+    }
+#else
+    rv = strcmp(sa, sb);
+#endif /* NLS && !NOSTRCOLL */
+
+    xfree((ptr_t) sa);
+    xfree((ptr_t) sb);
+
+    return rv;
+}
+
+#ifdef HASHBANG
+/*
+ * From: peter@zeus.dialix.oz.au (Peter Wemm)
+ * If exec() fails look first for a #! [word] [word] ....
+ * If it is, splice the header into the argument list and retry.
+ */
+#define HACKBUFSZ 1024		/* Max chars in #! vector */
+#define HACKVECSZ 128		/* Max words in #! vector */
+int
+hashbang(fd, vp)
+    int fd;
+    Char ***vp;
+{
+    unsigned char lbuf[HACKBUFSZ];
+    char *sargv[HACKVECSZ];
+    unsigned char *p, *ws;
+    int sargc = 0;
+
+    if (read(fd, (char *) lbuf, HACKBUFSZ) <= 0)
+	return -1;
+
+    ws = 0;	/* word started = 0 */
+
+    for (p = lbuf; p < &lbuf[HACKBUFSZ]; )
+	switch (*p) {
+	case ' ':
+	case '\t':
+	    if (ws) {	/* a blank after a word.. save it */
+		*p = '\0';
+		if (sargc < HACKVECSZ - 1)
+		    sargv[sargc++] = ws;
+		ws = NULL;
+	    }
+	    p++;
+	    continue;
+
+	case '\0':	/* Whoa!! what the hell happened */
+	    return -1;
+
+	case '\n':	/* The end of the line. */
+	    if (ws) {	/* terminate the last word */
+		*p = '\0';
+		if (sargc < HACKVECSZ - 1)
+		    sargv[sargc++] = ws;
+		sargv[sargc] = NULL;
+		ws = NULL;
+		*vp = blk2short(sargv);
+		return 0;
+	    }
+	    else
+		return -1;
+
+	default:
+	    if (!ws)	/* Start a new word? */
+		ws = p; 
+	    p++;
+	    break;
+	}
+    return -1;
+}
+#endif /* HASHBANG */
+
+#ifdef REMOTEHOST
+
+#ifdef ISC
+# undef MAXHOSTNAMELEN	/* Busted headers? */
+#endif
+
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <sys/socket.h>
+
+static sigret_t
+palarm(snum)
+    int snum;
+{
+    USE(snum);
+#ifdef UNRELSIGS
+    if (snum)
+	(void) sigset(snum, SIG_IGN);
+#endif /* UNRELSIGS */
+    (void) alarm(0);
+
+#ifndef SIGVOID
+    return (snum);
+#endif
+}
+
+/*
+ * From: <lesv@ppvku.ericsson.se> (Lennart Svensson)
+ */
+void 
+remotehost()
+{
+    char *host = NULL;
+    struct hostent* hp;
+    struct sockaddr_in saddr;
+    int len = sizeof(struct sockaddr_in);
+
+    /* Don't get stuck if the resolver does not work! */
+    sigret_t (*osig)() = signal(SIGALRM, palarm);
+    (void) alarm(2);
+
+    if (getpeername(SHIN, (struct sockaddr *) &saddr, &len) != -1) {
+	if ((hp = gethostbyaddr((char *)&saddr.sin_addr, len, AF_INET)) != NULL)
+	    host = hp->h_name;
+	else
+	    host = inet_ntoa(saddr.sin_addr);
+    }
+#ifdef UTHOST
+    else {
+	char *ptr, *sptr;
+	char *name = utmphost();
+	if (name != NULL) {
+	    /* Look for host:display.screen */
+	    if ((sptr = strchr(name, ':')) != NULL)
+		*sptr = '\0';
+	    if ((hp = gethostbyname(name)) == NULL) {
+		/* Try again eliminating the trailing domain */
+		if ((ptr = strchr(name, '.')) != NULL) {
+		    *ptr = '\0';
+		    if ((hp = gethostbyname(name)) != NULL)
+			host = hp->h_name;
+		    *ptr = '.';
+		}
+	    }
+	    else
+		host = hp->h_name;
+	    if (sptr)
+		*sptr = ':';
+	}
+    }
+#endif
+    (void) alarm(0);
+    (void) signal(SIGALRM, osig);
+
+    if (host)
+	tsetenv(STRREMOTEHOST, str2short(host));
+}
+#endif /* REMOTEHOST */
