@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.04/RCS/tc.sched.c,v 3.9 1993/07/03 23:47:53 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.05/RCS/tc.sched.c,v 3.10 1994/05/26 13:11:20 christos Exp $ */
 /*
  * tc.sched.c: Scheduled command execution
  *
@@ -38,7 +38,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.sched.c,v 3.9 1993/07/03 23:47:53 christos Exp $")
+RCSID("$Id: tc.sched.c,v 3.10 1994/05/26 13:11:20 christos Exp $")
 
 #include "ed.h"
 #include "tc.h"
@@ -73,7 +73,6 @@ dosched(v, c)
     Char   *cp;
     bool    relative;		/* time specified as +hh:mm */
     struct tm *ltp;
-    char   *timeline;
 
     USE(c);
 /* This is a major kludge because of a gcc linker  */
@@ -87,13 +86,17 @@ dosched(v, c)
     v++;
     cp = *v++;
     if (cp == NULL) {
+	Char   *fmt;
+	if ((fmt = varval(STRsched)) == STRNULL)
+	    fmt = str2short("%h\t%T\t%R\n");
 	/* print list of scheduled events */
 	for (count = 1, tp = sched_ptr; tp; count++, tp = tp->t_next) {
-	    timeline = (char *) ctime(&tp->t_when);
-	    timeline[16] = '\0';
-	    xprintf("%6d\t%s\t", count, timeline);
-	    blkpr(tp->t_lex);
-	    xputchar('\n');
+	    Char buf[BUFSIZE], sbuf[BUFSIZE], *cp;
+	    blkexpand(tp->t_lex, buf);
+	    tprintf(FMT_SCHED, sbuf, fmt, sizeof(sbuf), 
+		    short2str(buf), tp->t_when, (ptr_t) &count);
+	    for (cp = sbuf; *cp;)
+		xputchar(*cp++);
 	}
 	return;
     }
